@@ -1,11 +1,13 @@
 # freebsd-lib.pl
 # Mount table functions for freebsd
 
-$uname_release = `uname -r`;
+$uname_release = &backquote_command("uname -r");
 if (&has_command("mount_smbfs")) {
 	$smbfs_support = 1;
 	$nsmb_conf = "/etc/nsmb.conf";
 	}
+$ide_device_prefix = $uname_release > 9 ? "ada" : "ad";
+&foreign_require("bsdfdisk");
 
 # Return information about a filesystem, in the form:
 #  directory, device, type, options, fsck_order, mount_at_boot
@@ -17,7 +19,7 @@ sub list_mounts
 local(@rv, @p, @o, $_, $i, $j); $i = 0;
 
 # Get /etc/fstab mounts
-open(FSTAB, $config{'fstab_file'});
+open(FSTAB, "<".$config{'fstab_file'});
 while(<FSTAB>) {
 	local(@o, $at_boot);
 	chop; s/#.*$//g;
@@ -26,7 +28,7 @@ while(<FSTAB>) {
 	if ($p[2] eq "proc" || $p[2] eq "procfs") { $p[2] = $p[0] = "proc"; }
 	elsif ($p[2] eq "swap") { $p[1] = "swap"; }
 	elsif ($p[2] eq "smbfs") {
-		# Need to get nsmb.conf options, covert share and extract user
+		# Need to get nsmb.conf options, convert share and extract user
 		$p[0] = lc($p[0]);
 		local $noptions = &read_nsmb($p[0]);
 		local %options;
@@ -45,7 +47,7 @@ while(<FSTAB>) {
 	$rv[$i]->[5] = "yes";
 	@o = split(/,/ , $p[3] eq "defaults" ? "" : $p[3]);
 	if (($j = &indexof("noauto", @o)) >= 0) {
-		# filesytem is not mounted at boot
+		# filesystem is not mounted at boot
 		splice(@o, $j, 1);
 		$rv[$i]->[5] = "no";
 		}
@@ -115,7 +117,7 @@ local($i, @fstab, $line, $opts, $j, @amd);
 $i = 0;
 
 # Update fstab file
-open(FSTAB, $config{fstab_file});
+open(FSTAB, "<".$config{fstab_file});
 @fstab = <FSTAB>;
 close(FSTAB);
 &open_tempfile(FSTAB, "> $config{fstab_file}");
@@ -141,7 +143,7 @@ local($i, @fstab, $line, $opts, $j, @amd);
 $i = 0;
 
 # Update fstab file
-open(FSTAB, $config{fstab_file});
+open(FSTAB, "<".$config{fstab_file});
 @fstab = <FSTAB>;
 close(FSTAB);
 &open_tempfile(FSTAB, "> $config{fstab_file}");
@@ -168,7 +170,7 @@ sub list_mounted
 {
 # get the list of mounted filesystems
 local(@rv, $_);
-local $cmd = $uname_release =~ /^[789]\.[0-9]/ ? "freebsd-mounts-7" :
+local $cmd = $uname_release =~ /^(\d+)\.[0-9]/ && $1 > 6 ? "freebsd-mounts-7" :
 	     $uname_release =~ /^[56]\.[0-9]/ ? "freebsd-mounts-5" :
 	     $uname_release =~ /^4\.[0-9]/ ? "freebsd-mounts-4" :
 	     $uname_release =~ /^3\.[1-9]/ ? "freebsd-mounts-3" :
@@ -180,7 +182,7 @@ while(<CMD>) {
 	if ($p[2] eq "procfs" || $p[1] eq "procfs") { $p[1] = $p[2] = "proc"; }
 	elsif ($p[2] eq "mfs") { $p[1] =~ s/:.*$//; }
 	elsif ($p[2] eq "smbfs") {
-		# Need to get nsmb.conf options, covert share and extract user
+		# Need to get nsmb.conf options, convert share and extract user
 		$p[1] = lc($p[1]);
 		local $noptions = &read_nsmb($p[1]);
 		local %options;
@@ -321,8 +323,8 @@ if (&get_mounted($_[1], "*") < 0) { return (); }
 if ($_[0] eq "proc" || $_[0] eq "swap") { return (); }
 my $out;
 &execute_command("df -k ".quotemeta($_[1]), undef, \$out, undef, 0, 1);
-if ($out =~ /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/) {
-	return ($1, $2);
+if ($out =~ /Mounted on\n\S+\s+(?<total>\S+)\s+(?<used>\S+)\s+(?<free>\S+)\s+(?<percent>\d+)/) {
+	return ("$+{total}", "$+{free}", "$+{used}", "$+{percent}");
 	}
 else {
 	return ( );
@@ -409,34 +411,18 @@ else {
 		# Disk-based filesystem
 		$msg = &fstype_name($type);
 		}
-	local ($disk_dev, $ide_t, $ide_s, $ide_p, $scsi_t, $scsi_s, $scsi_p);
-	if ($loc =~ /^\/dev\/ad(\d)s(\d)([a-z]*)$/) {
-		$disk_dev = 0; $ide_t = $1; $ide_s = $2; $ide_p = $3;
-		}
-	elsif ($loc =~ /^\/dev\/da(\d)s(\d)([a-z]*)$/) {
-		$disk_dev = 1; $scsi_t = $1; $scsi_s = $2; $scsi_p = $3;
-		}
-	else { $disk_dev = 2; }
 
+	# Generate disk selection options
+	my @opts;
+	my $found;
+	my $sel = &bsdfdisk::partition_select(
+		"disk_select", $loc, 3, \$found);
+	push(@opts, [ 0, $text{'freebsd_select'}, $sel ]);
+	push(@opts, [ 1, $text{'freebsd_other'},
+		      &ui_textbox("dev_path", $found ? "" : $loc, 40).
+		      " ".&file_chooser_button("dev_path", 0) ]);
 	print &ui_table_row($msg,
-		&ui_radio_table("disk_dev", $disk_dev,
-		  [ [ 0, $text{'freebsd_ide'},
-		      $text{'freebsd_device'}." ".
-		        &ui_textbox("ide_t", $ide_t, 4)." ".
-		      $text{'freebsd_slice'}." ".
-		        &ui_textbox("ide_s", $ide_s, 4)." ".
-		      $text{'freebsd_part'}." ".
-		        &ui_textbox("ide_p", $ide_p, 4) ],
-		    [ 1, $text{'freebsd_scsi'},
-		      $text{'freebsd_device'}." ".
-		        &ui_textbox("scsi_t", $scsi_t, 4)." ".
-		      $text{'freebsd_slice'}." ".
-		        &ui_textbox("scsi_s", $scsi_s, 4)." ".
-		      $text{'freebsd_part'}." ".
-		        &ui_textbox("scsi_p", $scsi_p, 4) ],
-		    [ 2, $text{'freebsd_other'},
-		      &ui_textbox("dev_path", $disk_dev == 2 ? $loc : "", 40).
-		      " ".&file_chooser_button("dev_path", 0) ] ]));
+		&ui_radio_table("disk_dev", $found ? 0 : 1, \@opts));
 	}
 }
 
@@ -696,25 +682,28 @@ elsif ($_[0] eq "smbfs") {
 sub check_location
 {
 if ($_[0] eq "nfs") {
-	local($out, $temp, $mout, $dirlist);
+	local($out, $temp, $mout, $dirlist, $qnfs_host, $qnfs_dir, $qtemp);
+	$qnfs_host = quotemeta($in{nfs_host});
 
-	# Use ping and showmount to see if the host exists and is up
-	if ($in{nfs_host} !~ /^\S+$/) {
-		&error("'$in{nfs_host}' is not a valid hostname");
-		}
-	&execute_command("ping -c 1 '$in{nfs_host}'", undef, \$out, \$out);
-	if ($out =~ /unknown host/i) {
-		&error("The host '$in{nfs_host}' does not exist");
-		}
-	elsif ($out =~ /100\% packet loss/) {
-		&error("The host '$in{nfs_host}' is down");
-		}
-	&execute_command("showmount -e '$in{nfs_host}'", undef, \$out, \$out);
-	if ($out =~ /Unable to receive/) {
-		&error("The host '$in{nfs_host}' does not support NFS");
-		}
-	elsif ($?) {
-		&error("Failed to get mount list : $out");
+	if ($config{'nfs_check'}) {
+		# Use ping and showmount to see if the host exists and is up
+		if ($in{nfs_host} !~ /^\S+$/) {
+			&error("'$in{nfs_host}' is not a valid hostname");
+			}
+		&execute_command("ping -c 1 $qnfs_host", undef, \$out, \$out);
+		if ($out =~ /unknown host/i) {
+			&error("The host '$in{nfs_host}' does not exist");
+			}
+		elsif ($out =~ /100\% packet loss/) {
+			&error("The host '$in{nfs_host}' is down");
+			}
+		&execute_command("showmount -e $qnfs_host", undef, \$out, \$out);
+		if ($out =~ /Unable to receive/) {
+			&error("The host '$in{nfs_host}' does not support NFS");
+			}
+		elsif ($?) {
+			&error("Failed to get mount list : $out");
+			}
 		}
 
 	# Validate directory name
@@ -730,7 +719,9 @@ if ($_[0] eq "nfs") {
 	# Try a test mount to see if filesystem is available
 	$temp = &transname();
 	&make_dir($temp, 0755);
-	&execute_command("mount $in{nfs_host}:$in{nfs_dir} $temp",
+	$qnfs_dir = quotemeta($in{nfs_dir});
+	$qtemp = quotemeta($temp);
+	&execute_command("mount $qnfs_host:$qnfs_dir $qtemp",
 			 undef, \$mout, \$mout);
 	if ($mout =~ /No such file or directory/) {
 		&error("The directory '$in{nfs_dir}' does not exist on the ".
@@ -758,25 +749,13 @@ elsif ($_[0] eq "smbfs") {
 else {
 	# This is some kind of disk-based filesystem.. get the device name
 	if ($in{'disk_dev'} == 0) {
-		$in{'ide_t'} =~ /^\d+$/ ||
-			&error("'$in{ide_t}' is not a valid device number");
-		$in{'ide_s'} =~ /^\d+$/ ||
-			&error("'$in{ide_s}' is not a valid slice number");
-		$in{'ide_p'} =~ /^[a-z]*$/ ||
-			&error("'$in{ide_p}' is not a valid partition letter");
-		$dv = "/dev/ad$in{ide_t}s$in{ide_s}$in{ide_p}";
-		}
-	elsif ($in{'disk_dev'} == 1) {
-		$in{'scsi_t'} =~ /^\d+$/ ||
-			&error("'$in{scsi_t}' is not a valid device number");
-		$in{'scsi_s'} =~ /^\d+$/ ||
-			&error("'$in{scsi_s}' is not a valid slice number");
-		$in{'scsi_p'} =~ /^[a-z]*$/ ||
-			&error("'$in{scsi_p}' is not a valid partition letter");
-		$dv = "/dev/da$in{scsi_t}s$in{scsi_s}$in{scsi_p}";
+		# From menu
+		$dv = $in{'disk_select'};
 		}
 	else {
+		# Manually entered
 		$dv = $in{'dev_path'};
+		$dv =~ /^\// || &error($text{'freebsd_edevpath'});
 		}
 
 	# If the device entered is a symlink, follow it
@@ -788,7 +767,7 @@ else {
 		}
 
 	# Check if the device actually exists and uses the right filesystem
-	(-r $dv) || &error("The device file '$dv' does not exist");
+	(-r $dv) || &error(&text('freebsd_edevfile', $dv));
 	return $dv;
 	}
 }
@@ -942,13 +921,14 @@ return &join_options();
 # Calls dd and mkswap to setup a swap file
 sub create_swap
 {
-local($out, $bl);
+local($out, $bl, $qfile);
+$qfile = quotemeta($_[0]);
 $bl = $_[1] * ($_[2] eq "t" ? 1024*1024*1024 :
 	       $_[2] eq "g" ? 1024*1024 :
 	       $_[2] eq "m" ? 1024 : 1);
-$out = &backquote_logged("dd if=/dev/zero of=$_[0] bs=1024 count=$bl 2>&1");
+$out = &backquote_logged("dd if=/dev/zero of=$qfile bs=1024 count=$bl 2>&1");
 if ($?) { return "dd failed : $out"; }
-$out = &backquote_logged("mkswap $_[0] $bl 2>&1");
+$out = &backquote_logged("mkswap $qfile $bl 2>&1");
 if ($?) { return "mkswap failed : $out"; }
 &system_logged("sync >/dev/null 2>&1");
 return 0;
@@ -983,7 +963,8 @@ return "255.255.255.255";
 
 sub device_name
 {
-return $_[0];
+my ($dev) = @_;
+return &bsdfdisk::partition_description($dev);
 }
 
 sub files_to_lock
@@ -998,7 +979,7 @@ sub get_nsmb_conf
 local $conf;
 local $insection = 0;
 local $lnum = 0;
-open(CONF, $nsmb_conf);
+open(CONF, "<".$nsmb_conf);
 while(<CONF>) {
 	s/\r|\n//g;
 	s/^\s*#.*$//;
